@@ -11,7 +11,6 @@ to Excel files.
 from .utils import TimeSeries
 import sciris as sc
 from xlsxwriter.utility import xl_rowcol_to_cell as xlrc
-import openpyxl
 from .excel import cell_get_string, standard_formats, read_tables, TimeDependentValuesEntry, TimeDependentConnections, apply_widths, update_widths, validate_category
 import xlsxwriter as xw
 import io
@@ -23,6 +22,7 @@ from collections import defaultdict
 
 _DEFAULT_PROVENANCE = 'Framework-supplied default'
 
+
 class ProjectData(sc.prettyobj):
     """
     Store project data: class-equivalent of Databooks
@@ -318,6 +318,8 @@ class ProjectData(sc.prettyobj):
         # e.g. the fact that 'Alive' is a Characteristic is stored in the Framework and Data but not in the Databook. So for example, we read in
         # a TDVE table called 'Alive', but it needs to be stored in data.specs['charac']['ch_alive'] and the 'charac' and 'ch_alive' are only available in the Framework
 
+        import openpyxl
+
         self = ProjectData(framework=framework)
 
         if not isinstance(spreadsheet, sc.Spreadsheet):
@@ -580,7 +582,7 @@ class ProjectData(sc.prettyobj):
         """
 
         ss = self.to_spreadsheet()
-        ss.save(fname + '.xlsx' if not fname.endswith('.xlsx') else fname)
+        ss.save(fname)
 
     def add_pop(self, code_name: str, full_name: str, pop_type: str = None) -> None:
         """
