@@ -23,6 +23,7 @@ import atomica
 import types
 import numpy as np
 import pandas as pd
+from collections import defaultdict
 
 __all__ = ['migration', 'migrate', 'register_migration']
 
@@ -752,3 +753,10 @@ def _add_tdc_tdve_attributes(proj):
             tdc.attributes = {}
             tdc.ts_attributes = {}
     return proj
+
+
+@migration('Project', '1.19.0', '1.20.0', 'Framework.transitions is a defaultdict')
+def _framework_transitions_defaultdict(proj):
+    for fw in all_frameworks(proj):
+        fw.transitions = defaultdict(list, fw.transitions)
+    return proj
