@@ -9,9 +9,9 @@ set of programs, respectively.
 
 import io
 from datetime import timezone
+from pathlib import Path
 
 import numpy as np
-import openpyxl
 import xlsxwriter as xw
 from numpy import inf, array, exp, minimum
 from xlsxwriter.utility import xl_rowcol_to_cell as xlrc
@@ -453,6 +453,7 @@ class ProgramSet(NamedItem):
         :return: A :class:`ProgramSet`
 
         """
+        import openpyxl
 
         framework, data = ProgramSet._normalize_inputs(framework, data, project)
 
@@ -460,7 +461,7 @@ class ProgramSet(NamedItem):
         self = ProgramSet(name=name, framework=framework, data=data)
 
         # Create and load spreadsheet
-        if sc.isstring(spreadsheet):
+        if not isinstance(spreadsheet, sc.Spreadsheet):
             spreadsheet = sc.Spreadsheet(spreadsheet)
 
         workbook = openpyxl.load_workbook(spreadsheet.tofile(), read_only=True, data_only=True)  # Load in read-only mode for performance, since we don't parse comments etc.
@@ -518,12 +519,9 @@ class ProgramSet(NamedItem):
         spreadsheet = sc.Spreadsheet(f)  # Wrap it in a spreadsheet instance
         return spreadsheet
 
-    def save(self, filename=None, folder=None) -> str:
-        # Shortcut for saving to disk - FE RPC will probably use `to_spreadsheet()` but BE users will probably use `save()`
-        full_path = sc.makefilepath(filename=filename, folder=folder, default='Programs', ext='xlsx')
+    def save(self, fname):
         ss = self.to_spreadsheet()
-        ss.save(full_path)
-        return full_path
+        ss.save(fname)
 
     def _read_targeting(self, sheet, framework) -> None:
         # This function reads a targeting sheet and instantiates all of the programs with appropriate targets, putting them
