@@ -19,18 +19,17 @@ In addition, a project contains:
 """
 
 from .version import version, gitinfo
-from .calibration import perform_autofit
+from .calibration import calibrate
 from .data import ProjectData
 from .framework import ProjectFramework
 from .model import run_model
 from .parameters import ParameterSet
 
-from .programs import ProgramSet, ProgramInstructions
+from .programs import ProgramSet
 from .scenarios import Scenario, ParameterScenario, CombinedScenario, BudgetScenario, CoverageScenario
 from .optimization import Optimization, optimize, InvalidInitialConditions
 from .system import logger
-from .cascade import sanitize_cascade
-from .utils import NDict, evaluate_plot_string, NamedItem, TimeSeries
+from .utils import NDict, evaluate_plot_string, NamedItem, parallel_progress, Quiet
 from .plotting import PlotData, plot_series
 from .results import Result
 from .migration import migrate
@@ -39,6 +38,7 @@ import numpy as np
 import tqdm
 import logging
 from datetime import timezone
+import functools
 
 
 class ProjectSettings(object):
@@ -148,13 +148,13 @@ class Project(NamedItem):
     def __init__(self, name="default", framework=None, databook=None, do_run=True, **kwargs):
         NamedItem.__init__(self, name)
 
-        if sc.isstring(framework) or isinstance(framework, sc.Spreadsheet):
-            self.framework = ProjectFramework(inputs=framework)
-        elif isinstance(framework, ProjectFramework):
+        if isinstance(framework, ProjectFramework):
             self.framework = framework
-        else:
+        elif framework is None:
             logger.warning('Project was constructed without a Framework - a framework should be provided')
             self.framework = None
+        else:
+            self.framework = ProjectFramework(inputs=framework)
 
         # Define the structure sets
         self.parsets = NDict()
@@ -224,7 +224,7 @@ class Project(NamedItem):
     #######################################################################################################
     # Methods for I/O and spreadsheet loading
     #######################################################################################################
-    def create_databook(self, databook_path=None, num_pops=1, num_transfers=0, num_interpops=0, data_start=2000.0, data_end=2020.0, data_dt=1.0):
+    def create_databook(self, databook_path=None, num_pops=1, num_transfers=0, data_start=2000.0, data_end=2020.0, data_dt=1.0):
         """ Generate an empty data-input Excel spreadsheet corresponding to the framework of this project. """
         if databook_path is None:
             databook_path = "./databook_" + self.name + ".xlsx"
@@ -232,25 +232,29 @@ class Project(NamedItem):
         data.save(databook_path)
         return data
 
-    def load_databook(self, databook_path=None, make_default_parset=True, do_run=True) -> None:
+    def load_databook(self, databook_path, make_default_parset: bool = True, do_run: bool = True) -> None:
         """
         Load a data spreadsheet
 
-        :param databook_path: a path string, which will load a file from disk, or an sc.Spreadsheet
-                                containing the contents of a databook
-        :param make_default_parset: If True, a Parset called "default" will be immediately created from the
-                                    newly-added data
+        :param databook_path: Databook input. Supported inputs are:
+                                - A path string, which will load a file from disk
+                                - An `sc.Spreadsheet` containing the contents of a databook
+                                - A `ProjectData` instance
+        :param make_default_parset: If True, a Parset called "default" will be immediately created from the newly-added data
         :param do_run: If True, a simulation will be run using the new parset
 
         """
 
-        if sc.isstring(databook_path):
-            full_path = sc.makefilepath(filename=databook_path, default=self.name, ext='xlsx', makedirs=False)
-            databook_spreadsheet = sc.Spreadsheet(full_path)
+        if isinstance(databook_path, ProjectData):
+            # If the user passed in a ProjectData instance, use it directly
+            data = databook_path
+            databook = data.to_spreadsheet()
         else:
-            databook_spreadsheet = databook_path
-
-        data = ProjectData.from_spreadsheet(databook_spreadsheet, self.framework)
+            if isinstance(databook_path, sc.Spreadsheet):
+                databook = databook_path
+            else:
+                databook = sc.Spreadsheet(databook_path)
+            data = ProjectData.from_spreadsheet(databook, self.framework)
 
         # If there are existing progsets, make sure the new data is consistent with them
         if self.progsets:
@@ -260,7 +264,7 @@ class Project(NamedItem):
 
         self.data = data
         self.data.validate(self.framework)  # Make sure the data is suitable for use in the Project (as opposed to just manipulating the databook)
-        self.databook = sc.dcp(databook_spreadsheet)  # Actually a shallow copy is fine here because sc.Spreadsheet contains no mutable properties
+        self.databook = sc.dcp(databook)
         self.modified = sc.now(utc=True)
         self.settings.update_time_vector(start=self.data.start_year)  # Align sim start year with data start year.
 
@@ -280,7 +284,7 @@ class Project(NamedItem):
         self.parsets.append(ParameterSet(self.framework, self.data, name))
         return self.parsets[name]
 
-    def make_progbook(self, progbook_path: str=None, progs=None, data_start: float=None, data_end: float=None) -> str:
+    def make_progbook(self, progbook_path: str = None, progs=None, data_start: float = None, data_end: float = None) -> str:
         """
         Save a blank program book
 
@@ -503,7 +507,7 @@ class Project(NamedItem):
 
         return result
 
-    def run_sampled_sims(self, parset, progset=None, progset_instructions=None, result_names=None, n_samples: int = 1, parallel=False, max_attempts=None) -> list:
+    def run_sampled_sims(self, parset, progset=None, progset_instructions=None, result_names=None, n_samples: int = 1, parallel=False, max_attempts=None, num_workers=None) -> list:
         """
         Run sampled simulations
 
@@ -511,14 +515,10 @@ class Project(NamedItem):
         several reasons
 
         - To avoid inadvertantly blowing up the size of the project, `run_sampled_sims` does not support automatic result saving
-        - `run_sim` always returns a `Result` - if rolled into one functions, the return type would not be predictable
+        - `run_sim` always returns a `Result` - if rolled into one function, the return type would not be predictable
         - `run_sim` only takes in a single ``ProgramInstructions`` and ``result_name`` whereas `run_sampled_sims` supports iteration
            over multiple instructions
 
-
-        This method is different from proj.run_sim(samples=n_samples) in two ways
-        -
-
         The other common scenario is having multiple results
 
         :param n_samples: An integer number of samples
@@ -530,6 +530,7 @@ class Project(NamedItem):
                              containing a single element if not using programs.
         :param parallel: If True, run simulations in parallel (on Windows, must have ``if __name__ == '__main__'`` gating the calling code)
         :param max_attempts: Number of retry attempts for bad initializations
+        :param num_workers: If ``parallel`` is True, this determines the number of parallel workers to use (default is usually number of CPUs)
         :return: A list of Results that can be passed to `Ensemble.update()`. If multiple instructions are provided, the return value of this
                  function will be a list of lists, where the inner list iterates over different instructions for the same parset/progset samples.
                  It is expected in that case that the Ensemble's mapping function would take in a list of results
@@ -551,22 +552,19 @@ class Project(NamedItem):
             result_names = sc.promotetolist(result_names)
             assert (len(result_names) == 1 and not progset) or (len(progset_instructions) == len(result_names)), "Number of result names must match number of instructions"
 
-        original_level = logger.getEffectiveLevel()
-        logger.setLevel(logging.WARNING)  # Never print debug messages inside the sampling loop - note that depending on the platform, this may apply within `sc.parallelize`
+        show_progress = n_samples > 1 and logger.getEffectiveLevel() <= logging.INFO
 
-        if n_samples == 1:
-            results = [_run_sampled_sim(self, parset, progset, progset_instructions, result_names, max_attempts=max_attempts)]
-        elif parallel:
-            # NB. The calling code must be wrapped in a 'if __name__ == '__main__' if on Windows
-            results = sc.parallelize(_run_sampled_sim, iterarg=n_samples, kwargs={'proj': self, 'parset': parset, 'progset': progset, 'progset_instructions': progset_instructions, 'result_names': result_names, 'max_attempts': max_attempts})
-        else:
-            if original_level <= logging.INFO:
-                # Print the progress bar if the logging level was INFO or lower
+        if parallel:
+            fcn = functools.partial(_run_sampled_sim, proj=self, parset=parset, progset=progset, progset_instructions=progset_instructions, result_names=result_names, max_attempts=max_attempts)
+            results = parallel_progress(fcn, n_samples, show_progress=show_progress, num_workers=num_workers)
+        elif show_progress:
+            # Print the progress bar if the logging level was INFO or lower
+            # This means that the user can still set the logging level higher e.g. WARNING to suppress output from Atomica in general
+            # (including any progress bars)
+            with Quiet():
                 results = [_run_sampled_sim(self, parset, progset, progset_instructions, result_names, max_attempts=max_attempts) for _ in tqdm.trange(n_samples)]
-            else:
-                results = [_run_sampled_sim(self, parset, progset, progset_instructions, result_names, max_attempts=max_attempts) for _ in range(n_samples)]
-
-        logger.setLevel(original_level)  # Reset the logger
+        else:
+            results = [_run_sampled_sim(self, parset, progset, progset_instructions, result_names, max_attempts=max_attempts) for _ in range(n_samples)]
 
         return results
 
@@ -601,7 +599,9 @@ class Project(NamedItem):
 
         if parset is None:
             parset = -1
-        parset = self.parsets[parset]
+        elif not isinstance(parset, ParameterSet):
+            parset = self.parsets[parset]
+
         if new_name is None:
             new_name = parset.name + ' (auto-calibrated)'
         if adjustables is None:
@@ -617,8 +617,8 @@ class Project(NamedItem):
         for index, measurable in enumerate(measurables):
             if sc.isstring(measurable):  # Assume that a parameter name was passed in if not a tuple.
                 measurables[index] = (measurable, None, default_weight, default_metric)
-        new_parset = perform_autofit(project=self, parset=parset,
-                                     pars_to_adjust=adjustables, output_quantities=measurables, max_time=max_time)
+        new_parset = calibrate(project=self, parset=parset,
+                               pars_to_adjust=adjustables, output_quantities=measurables, max_time=max_time)
         new_parset.name = new_name  # The new parset is a calibrated copy of the old, so change id.
         if save_to_project:
             self.parsets.append(new_parset)
