@@ -13,6 +13,7 @@ import pandas as pd
 from scipy import stats
 import tqdm
 import logging
+from pathlib import Path
 
 import sciris as sc
 from .excel import standard_formats
@@ -433,7 +434,8 @@ class Result(NamedItem):
 
         # Optionally save it
         if filename is not None:
-            df.T.to_excel(filename + '.xlsx' if not filename.endswith('.xlsx') else filename)
+            output_fname = Path(filename).with_suffix('.xlsx').resolve()
+            df.T.to_excel(output_fname)
 
         return df
 
@@ -517,7 +519,8 @@ def _filter_pops_by_output(result, output) -> list:
         raise Exception('Could not determine population type')
 
     matching_pops = {x.pop.name for x in vars}
-    return [x for x in result.pop_names if x in matching_pops] # Maintain original population order
+    return [x for x in result.pop_names if x in matching_pops]  # Maintain original population order
+
 
 def export_results(results, filename=None, output_ordering=('output', 'result', 'pop'),
                    cascade_ordering=('pop', 'result', 'stage'), program_ordering=('program', 'result', 'quantity')):
@@ -573,7 +576,7 @@ def export_results(results, filename=None, output_ordering=('output', 'result',
     new_tvals = np.arange(np.ceil(results[0].t[0]), np.floor(results[0].t[-1]) + 1)
 
     # Open the output file
-    output_fname = filename + '.xlsx' if not filename.endswith('.xlsx') else filename
+    output_fname = Path(filename).with_suffix('.xlsx').resolve()
     writer = pd.ExcelWriter(output_fname, engine='xlsxwriter')
     formats = standard_formats(writer.book)
 
