@@ -3,23 +3,40 @@ Define utility classes used throughout Atomica
 
 """
 
-import inspect
-import os
 import ast
-from bisect import bisect
-import numpy as np
-import scipy.interpolate
-import sciris as sc
+import inspect
 import itertools
-import zlib
+import logging
+import os
 import re
 import time
+import zlib
+from bisect import bisect
 from datetime import datetime
+from functools import partial
+from pathlib import Path
+
+import numpy as np
+import scipy.interpolate
+from tqdm import tqdm
+
+import sciris as sc
+from .system import logger
 
 
-def parent_dir():
-    # Return the parent directory of the file that called this function
-    return os.path.join(os.path.abspath(os.path.join(inspect.stack()[1][1], os.pardir)), '')
+def parent_dir() -> Path:
+    """
+    Return the parent directory of current file
+
+    This function returns a Path object for the folder containing the file that called this function
+    e.g. if the file on disk is `foo/bar/baz.py` and `baz.py` contains `x = at.parent_dir()` then
+    `x` would be an absolute path corresponding to `Path('foo/bar')`
+
+    :return: Path object to the directory containing calling file
+
+    """
+
+    return Path(inspect.stack()[1][1]).parent
 
 
 class NamedItem():
@@ -750,6 +767,7 @@ def fast_gitinfo(path):
     output = {'branch': gitbranch, 'hash': githash, 'date': gitdate}  # Assemble outupt
     return output
 
+
 def datetime_to_year(dt: datetime) -> float:
     """
     Convert a DateTime instance to decimal year
@@ -762,5 +780,176 @@ def datetime_to_year(dt: datetime) -> float:
     """
     # By Luke Davis from https://stackoverflow.com/a/42424261
     year_part = dt - datetime(year=dt.year, month=1, day=1)
-    year_length = datetime(year=dt.year+1, month=1, day=1) - datetime(year=dt.year, month=1, day=1)
-    return dt.year + year_part/year_length
\ No newline at end of file
+    year_length = datetime(year=dt.year + 1, month=1, day=1) - datetime(year=dt.year, month=1, day=1)
+    return dt.year + year_part / year_length
+
+
+def _worker_init() -> None:
+    """
+    Suppress output on parallel workers
+
+    A parallel worker should only ever print out warning output or higher
+    This function gets passed as an initializer to `multiprocessing.Pool`
+    to set the logger level locally on the workers
+
+    """
+
+    logger.setLevel(logging.WARNING)
+
+
+def parallel_progress(fcn, inputs, num_workers=None, show_progress=True) -> list:
+    """
+    Run a function in parallel with a optional single progress bar
+
+    The result is essentially equivalent to
+
+    >>> list(map(fcn, inputs))
+
+    But with execution in parallel and with a single progress bar being shown.
+    The Atomica logger level will be changed to hide output below the
+    WARNING level.
+
+    :param fcn: Function object to call, accepting one argument, OR a function with zero arguments in which
+                case inputs should be an integer
+    :param inputs: A collection of inputs that will each be passed to (list, array, etc.)
+                    OR a number, if the fcn() has no input arguments
+    :param num_workers: Number of processes, defaults to the number of CPUs
+    :return: An list of outputs
+
+    """
+
+    from multiprocessing import pool
+
+    pool = pool.Pool(num_workers, initializer=_worker_init)
+
+    results = [None]
+    if sc.isnumber(inputs):
+        results *= inputs
+        pbar = tqdm(total=inputs) if show_progress else None
+    else:
+        results *= len(inputs)
+        pbar = tqdm(total=len(inputs)) if show_progress else None
+
+    def callback(result, idx):
+        results[idx] = result
+        if show_progress:
+            pbar.update(1)
+
+    if sc.isnumber(inputs):
+        for i in range(inputs):
+            pool.apply_async(fcn, callback=partial(callback, idx=i))
+    else:
+        for i, x in enumerate(inputs):
+            pool.apply_async(fcn, args=(x,), callback=partial(callback, idx=i))
+
+    pool.close()
+    pool.join()
+
+    if show_progress:
+        pbar.close()
+
+    return results
+
+
+class Quiet():
+    """
+    Atomica quiet context
+
+    This object can be used in a `with/as` block to temporarily suppress
+    Atomica output, with the logger level reset at the end of the block
+
+
+    Example usage:
+
+    >>> with at.Quiet():
+    >>>     res = P.run_sim()
+
+    """
+
+    def __init__(self, show_warnings=True):
+        """
+        Initialize standard context
+
+        Initialization captures the current logger level at the time the context is created.
+
+        :param show_warnings: If True, logger will be temporarily set at WARNING level. If
+                              False, all output will be suppressed by setting logger above CRITICAL
+        """
+
+        self.previous_level = logger.getEffectiveLevel()
+        self.show_warnings = show_warnings
+
+    def __enter__(self):
+        if self.show_warnings:
+            logger.setLevel(logging.WARNING)
+        else:
+            logger.setLevel(logging.CRITICAL + 1)
+
+    def __exit__(self, *args):
+        logger.setLevel(self.previous_level)
+
+
+def start_logging(fname: str) -> None:
+    """
+    Log Atomica output to a file
+
+    This function automatically starts the file with a version record.
+
+    The file logger will potentially handle any log level, from 0 upwards.
+    Therefore both stdout and stderr can potentially be logged. The stdout
+    messages will be gated by the log level of `at.logger`. In practice,
+    this means that the log file generated here will replicate whatever is
+    being printed to the screen by the main Atomica process.
+
+    The key items that will NOT be captured in this file log are
+        - Output generated by a `print` statement. Instead of using `print`, use
+          `at.logger.info('message')` which will both print and log the output
+        - Output generated by worker processes e.g. if using `run_sampled_sims(parallel=True)`
+          This is because the workers can't all write to the same log file without overwriting
+          each other. Normal log output is typically suppressed anyway - however, stderr messages
+          from the workers will be printed to the console but not logged in the file. These messages
+          would typically corrupt the progress bar in the console anyway.
+
+    :param fname: File name to write log file to
+
+    """
+
+    for handler in logger.handlers:
+        if handler.name == 'atomica_file_handler':
+            # Logging has already been started, so we can return immediately
+            # Otherwise, output could be logged multiple times
+            return
+
+    from .version import gitinfo, version, versiondate  # Avoid circular import
+
+    sc.makefilepath(fname)
+    h = logging.FileHandler(fname, mode='w')
+    h.set_name('atomica_file_handler')
+    fmt = logging.Formatter("%(asctime)-20s %(message)s", datefmt='%d-%m-%y %H:%M:%S')
+    h.setFormatter(fmt)
+
+    logger.addHandler(h)
+
+    logger.critical(f'Atomica log file: {os.path.abspath(fname)}')
+    logger.critical('-' * 80)
+    logger.critical('Atomica %s (%s) -- (c) the Atomica development team' % (version, versiondate))  # Log with the highest level to make sure it appears in the log
+    if gitinfo['branch'] != 'N/A':
+        logger.critical('git branch: %s (%s)' % (gitinfo['branch'], gitinfo['hash'][0:8]))
+    logger.critical(datetime.now())
+    logger.critical('-' * 80)
+
+    # We also want uncaught exceptions (in the main process) to appear in the log
+    # Based on https://stackoverflow.com/a/57587758 by Nabs
+    def log_exception(exctype, value, traceback):
+        logger.error("UNCAUGHT EXCEPTION", exc_info=(exctype, value, traceback))
+
+    def attach_hook(hook_func, run_func):
+        def inner(*args, **kwargs):
+            local_args = run_func()
+            hook_func(*local_args)
+            return run_func(*args, **kwargs)
+        return inner
+
+    import sys
+    sys.exc_info = attach_hook(log_exception, sys.exc_info)
+    sys.excepthook = log_exception
